@@ -5,7 +5,7 @@ from typing import Dict, List
 
 from pandas import Timestamp
 
-from models.product import Product
+from napoleontoolbox.models.product import Product
 
 
 @dataclass
