@@ -211,6 +211,7 @@ class MultiLayerShortCutPerceptron(_ShortCutPerceptronBaseNeuralNet):
         self.n_layers =  len(layers)
         # Set input layer
         input_size = self.N
+        shortcut_size = X_shortcut.shape[1]
         for output_size in layers:
             # Set hidden layers
             layers_list += [torch.nn.Linear(
@@ -221,8 +222,10 @@ class MultiLayerShortCutPerceptron(_ShortCutPerceptronBaseNeuralNet):
             input_size = output_size
 
         # Set output layer
-        layers_list += [torch.nn.Linear(input_size, self.M, bias=bias)]
+        self.final_layer = torch.nn.Linear(input_size+shortcut_size,self.M, bias=bias)
+        #layers_list += [torch.nn.Linear(input_size, self.M, bias=bias)]
         self.layers = torch.nn.ModuleList(layers_list)
+#        self.final_layer = torch.nn.Linear(layers[-1] )
 
 
         # Set activation functions
@@ -248,6 +251,13 @@ class MultiLayerShortCutPerceptron(_ShortCutPerceptronBaseNeuralNet):
             x = layer(x)
             x = self.activation(x)
             self.handle_callback('after_layer',layer_index = layer_index+1, layer_activ_mean=x.data.mean().item(), layer_activ_std = x.data.std().item())
+
+        x = torch.cat([x, x_shortcut], dim=1)
+        x = self.final_layer(x)
+        x = self.activation(x)
+        self.handle_callback('after_layer', layer_index=layer_index + 1, layer_activ_mean=x.data.mean().item(),
+                             layer_activ_std=x.data.std().item())
+
         return x
 
 
@@ -335,9 +345,9 @@ class RollMultiLayerShortCutPerceptron(MultiLayerShortCutPerceptron, roller._Sho
     def _train(self, X, X_shortcut, y):
         return self.train_on(X=X,X_shortcut=X_shortcut, y=y)
 
-    def sub_predict(self, X):
+    def sub_predict(self, X, X_shortcut):
         """ Predict. """
-        return self.predict(X=X)
+        return self.predict(X=X,X_shortcut = X_shortcut)
 
     def eval_predictor_importance(self, features, actual_predictions, features_names, check = True):
         deep_explainer = shap.DeepExplainer(self, features)
@@ -404,10 +414,10 @@ class RollMultiLayerShortCutPerceptron(MultiLayerShortCutPerceptron, roller._Sho
         self.handle_callback('begin_fit')
         for eval_slice, test_slice in self:
             # Compute prediction on eval and test set
-            eval_prediction = self.sub_predict(self.X[eval_slice])
+            eval_prediction = self.sub_predict(self.X[eval_slice], self.X_shortcut[eval_slice])
             self.y_eval[eval_slice] = eval_prediction
             predictors = self.X[test_slice]
-            test_prediction = self.sub_predict(predictors)
+            test_prediction = self.sub_predict(self.X[test_slice], self.X_shortcut[test_slice])
             self.y_test[test_slice] = test_prediction
 
             ev = self.y_eval[eval_slice]
